// << ------------------------------------ >>
// << SLADE - SlayeR's LeetAss Doom Editor >>
// << By Simon Judd, 2004-05               >>
// << ------------------------------------ >>
// << tex_browse.cpp - Texture browser     >>
// << ------------------------------------ >>

// INCLUDES ------------------------------ >>
#include "main.h"
#include "draw.h"

// VARIABLES ----------------------------- >>
texture_t*		*atextures = 0;
stringlist_t	browser_textures;
DWORD			n_tex = 0;
int				tex_index = 0;
int				row_offset = 0;
int				n_rows = 0;
int				tex_width = 0;
string			browser_search;

CVAR(Int, browser_columns, 8, CVAR_SAVE)

// EXTERNAL VARIABLES -------------------- >>
extern TextureList tex_list;
EXTERN_CVAR(Int, vid_width)
EXTERN_CVAR(Int, vid_height)
extern rgba_t col_selbox, col_selbox_line;
extern bool mix_tex;

// setup_browser: Loads textures for browsing (flats/textures/both)
// ------------------------------------------------------------- >>
void setup_browser(bool textures, bool flats)
{
	browser_search = "";
	n_tex = 0;
	tex_width = (vid_width - 32) / browser_columns;
	n_rows = vid_height / tex_width;

	browser_textures.clear();

	if (mix_tex)
	{
		textures = true;
		flats = true;
	}

	if (textures)
		browser_textures.add("-");

	for (DWORD t = 0; t < tex_list.length(); t++)
	{
		if (tex_list.texture(t)->available)
		{
			if (tex_list.texture(t)->flat && flats)
				browser_textures.add(tex_list.texture(t)->name);
			else if (!tex_list.texture(t)->flat && textures)
				browser_textures.add(tex_list.texture(t)->name);
		}
	}

	browser_textures.sort();
}

// draw_browser: Draws the texture browser screen
// ------------------------------------------- >>
void draw_browser()
{
	int x = 0;
	int y = 0;
	int start = row_offset * browser_columns;
	int end = start + browser_columns * n_rows;
	int width = tex_width;

	//if (end > n_tex)
	//	end = n_tex;
	if (end > browser_textures.n_strings)
		end = browser_textures.n_strings;

	for (DWORD t = start; t < end; t++)
	{
		texture_t *tex = tex_list.get_tex(*browser_textures.strings[t]);

		if (t == tex_index)
		{
			draw_rect(rect_t(x * width + 17, y * width + 1, width - 2, width - 2, 0), col_selbox, true);
			draw_rect(rect_t(x * width + 17, y * width + 1, width - 2, width - 2, 0), col_selbox_line, false);
			draw_text(vid_width, vid_height - 8, COL_BLUE, 2, "Dimensions: %d x %d", tex->width, tex->height);
		}

		if (t > 0)
			draw_texture_scale(tex, x * width + 24, (y * width) + 8, width - 16, width - 16, COL_WHITE, true, true);

		draw_text(x * width + (width / 2) + 16, (y * width) + width - 8, COL_WHITE, 1, browser_textures.strings[t]->c_str());
		draw_text(0, vid_height - 8, COL_BLUE, 0, "Search: %s", browser_search.c_str());

		x++;

		if (x == browser_columns)
		{
			x = 0;
			y++;
		}
	}
}

// browser_showtex: Adjusts row_offset until the currently hilighted texture is on screen
// ----------------------------------------------------------------------------------- >>
void browser_showtex()
{
	int start = row_offset * browser_columns;
	int end = start + browser_columns * n_rows;

	if (end > browser_textures.n_strings)
		end = browser_textures.n_strings;

	if (tex_index < start)
	{
		row_offset--;
		browser_showtex();
	}

	if (tex_index >= end)
	{
		row_offset++;
		browser_showtex();
	}
}

// browser_changetex: Moves the selected texture index in a specific direction
// ------------------------------------------------------------------------ >>
void browser_changetex(int direction)
{
	if (direction == 0)
	{
		if (--tex_index < 0)
			tex_index = 0;
	}

	if (direction == 1)
	{
		if (++tex_index >= browser_textures.n_strings)
			tex_index = browser_textures.n_strings - 1;
	}

	if (direction == 2)
	{
		int current = tex_index;
		tex_index -= browser_columns;

		if (tex_index < 0)
			tex_index = current;
	}

	if (direction == 3)
	{
		int current = tex_index;
		tex_index += browser_columns;

		if (tex_index >= browser_textures.n_strings)
			tex_index = browser_textures.n_strings - 1;
	}

	browser_showtex();
}

// browser_findtex: Finds the first texture in the list beginning with the search string
// ---------------------------------------------------------------------------------- >>
void browser_findtex(string search)
{
	for (DWORD t = 0; t < browser_textures.n_strings; t++)
	{
		if (browser_textures.strings[t]->substr(0, search.length()) == search)
		{
			tex_index = t;
			browser_showtex();
			return;
		}
	}
}

// browser_gettex: Returns the name of the hilighted texture
// ------------------------------------------------------ >>
string browser_gettex()
{
	//return atextures[tex_index]->name;
	return *browser_textures.strings[tex_index];
}