
#include "memreader.h"
#include "args.h"

struct thingdef_t
{
	short		type;
	string		name;
	rgba_t		colour;
	short		size;
	string		args[5];
	string*		arg_types[5];
	string		spritename;
	bool		show_angle;
	bool		hanging;
	texture_t	*sprite;

	thingdef_t();

	void dump()
	{
		printf("Thing %d: \"%s\", %d, %d, %d, %d, %d\n", type, name.c_str(), colour.r, colour.g, colour.b, size, show_angle);
	}
};

struct thinggroup_t
{
	string		name;
	thingdef_t*	*defs;
	DWORD		n_defs;

	thinggroup_t()
	{
		name = "-";
		defs = NULL;
		n_defs = 0;
	}

	void add(thingdef_t *tdef)
	{
		n_defs++;
		defs = (thingdef_t **)realloc(defs, n_defs * sizeof(thingdef_t *));
		defs[n_defs - 1] = tdef;
	}
};

struct thing_t
{
	// Doom format
	short	x;
	short	y;
	short	type;
	short	flags;
	short	angle;

	// Hexen format extras
	WORD	tid;
	short	z;
	BYTE	special;
	BYTE	args[5];

	// Editor stuff
	bool hilighted, selected, moving;

	rgba_t		colour;
	short		radius;
	bool		draw_angle;
	thingdef_t	*tdef;

	thing_t();

	void toggle_select()
	{
		selected = !selected;
	}

	void select()
	{
		selected = true;
	}

	void deselect()
	{
		selected = false;
	}
};

// Doom flags
#define THING_EASY		0x0001
#define THING_MEDIUM	0x0002
#define THING_HARD		0x0004
#define THING_DEAF		0x0008
#define THING_MULTI		0x0010

// Extra hexen flags
#define THING_DORMANT		0x0010	// Also zdoom
#define THING_FIGHTER		0x0020
#define THING_CLERIC		0x0040
#define THING_MAGE			0x0080
#define THING_SINGLE		0x0100	// Also zdoom
#define THING_COOPERATIVE	0x0200	//     ''
#define THING_DEATHMATCH	0x0400	//     ''

// Extra zdoom flags
#define THING_TRANSLUCENT	0x0800
#define THING_INVISIBLE		0x1000
#define THING_FRIENDLY		0x2000
#define THING_STANDSTILL	0x4000

void parse_thing_group(MemReader *mr);
void update_map_things();
thingdef_t* get_tdef(short type);
void thing_settype(BYTE group, WORD item, DWORD thing);
void update_tdef_textures();